--[[
	Copyright 2010 Matthew Mullins (aka Baffler)
	
    This file is part of 'Quartz: Procs' (World of Warcraft addon).

    'Quartz: Procs' is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    'Quartz: Procs' is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with 'Quartz: Procs'.  If not, see <http://www.gnu.org/licenses/>.
]]

--[[
	* For next beta
	- Internal cooldowns for custom procs
	- Custom procs based on combat log events
	
	x Spells not working/added
	XX Force of Nature (treants)
	XX mage (combat log stuff)
	XX Sacred Shield (target) [self is added]
	XX Ardent Defender debuff
	XX Fix stacking procs (serep, maelstrom)
	
	X Add a link/or some way to configure the mirror module from the procs config
	X /procs to open config
	X fix combat log aura refresh (so arcaneblast(4) will refresh timer)
]]

local Quartz3 = LibStub("AceAddon-3.0"):GetAddon("Quartz3")
local MODNAME = "Procs"
local QuartzProcs = Quartz3:NewModule(MODNAME, "AceEvent-3.0")
local Mirror = Quartz3:GetModule("Mirror")
local ex = Mirror.ExternalTimers
local media = LibStub("LibSharedMedia-3.0")

function del(tbl)
	wipe(tbl)
end

local db

local defaults = {
	profile = {
		spells = {
			[GetSpellInfo(32182)] = {
				enabled = true, color = {1, 0.65, 0}, ptype = "custom",
			}, -- Heroism
			[GetSpellInfo(2825)] = {
				enabled = true, color = {1, 0.65, 0}, ptype = "custom",
			},-- Bloodlust
		},
	}
}

local procs = {}

function QuartzProcs:OnInitialize()
	self.db = Quartz3.db:RegisterNamespace(MODNAME, defaults)
	db = self.db.profile
	
	self:SetEnabledState(Quartz3:GetModuleEnabled(MODNAME))
	Quartz3:RegisterModuleOptions(MODNAME, getOptions, "Procs")
	
	Quartz3:RegisterChatCommand("procs", function()
		if (Quartz3.optFrames[MODNAME] ~= nil) then
			InterfaceOptionsFrame_OpenToCategory(Quartz3.optFrames[MODNAME])
		else
			print(MODNAME .. " options have not been setup!")
		end
	end)
	
	procs = self:returnProcList()
	
	for spellkey,spell in pairs(db.spells) do
		if (spell.ptype ~= nil) and (spell.ptype == "custom") then
			procs[spellkey] = db.spells[spellkey]
		end
	end
	
	self:checkTalentPoints()
end

function QuartzProcs:OnEnable()
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	self:RegisterEvent("UNIT_AURA")
	self:RegisterEvent("PLAYER_TALENT_UPDATE")
end

function QuartzProcs:OnDisable()

end


-- check talent points for paladin, to update timer lengths
function QuartzProcs:PLAYER_TALENT_UPDATE()
	local _, curClass = UnitClass("player")
	if (curClass == "PALADIN") then
		self:checkTalentPoints()
	end
end

-- get the divine guardian talent (to see how long to increase sacred shield buff)
function QuartzProcs:checkTalentPoints()
	local _, curClass = UnitClass("player")
	if (curClass == "PALADIN") then
		nameTalent, icon, tier, column, currRank, maxRank = GetTalentInfo(2,9); -- Divine Guardian
		
		if (procs ~= nil) and (procs.combatlog ~= nil) and (procs.combatlog["53601"] ~= nil)
		then
			--print("- "..nameTalent..": "..currRank.."/"..maxRank);
			procs.combatlog["53601"].dur = 30 + (30 *  (currRank * 0.5))
			--print("- dur = " .. procs.combatlog["53601"].dur)
		end
	end
end

function QuartzProcs:UNIT_AURA(event, unit)
	if (unit == "player") then
		self:CheckBuffs()
	end
end

function QuartzProcs:COMBAT_LOG_EVENT_UNFILTERED(event, ...)
	local timestamp, eventType, sourceGUID, sourceName, sourceFlags, destGUID, destName, destFlags = select(1, ...)
	
	if (destGUID == UnitGUID("player")) then --check if the player is receiving the buff
	
		local spellID, spellname, spellSchool = select(9, ...)
		if (spellID ~= nil) and (spellID ~= "") then spellID = tostring(spellID) end
		
		if (eventType == "SPELL_AURA_APPLIED") or (eventType == "SPELL_AURA_REFRESH") then
			if (procs ~= nil) and (procs.combatlog ~= nil) and (procs.combatlog[spellID] ~= nil)
				--and ((procs.combatlog[spellID].cast == nil) or (procs.combatlog[spellID].cast == false))
			then
				local spell = procs.combatlog[spellID]
				local expr = GetTime() + spell.dur
				local _,_,icon = GetSpellInfo(tonumber(spellID))
				local sdb = db.spells

				if ((sdb[spellID] == nil) or (sdb[spellID].enabled == nil) or (sdb[spellID].enabled == true))
				then
					local sound = "None"
					if ((sdb[spellID] ~= nil) and (sdb[spellID].sound ~= nil)) then
						sound = sdb[spellID].sound
					end
					
					local spellcolor = spell.color or {1, 0, 0}
					if ((sdb[spellID] ~= nil) and (sdb[spellID].color ~= nil)) then
						spellcolor = sdb[spellID].color
					end

					if (eventType == "SPELL_AURA_REFRESH") then
						-- refresh timer (used so like Arcane Blast(4) will stay at 4)
						self:refreshSpell(spell.name, spell.dur, expr, tonumber(spellID), spellcolor, sound, "combatlog")
					else
						self:createCustomTimer(spell.name, spell.dur, expr, tonumber(spellID), spellcolor, false, sound, "combatlog")
					end
				end
			end
		-- for spell stacking (so far this is only going to be used for ArcaneBlast debuff.. hmmm)
		elseif (eventType == "SPELL_AURA_APPLIED_DOSE") then
			if (procs ~= nil) and (procs.combatlog ~= nil) and (procs.combatlog[spellID] ~= nil) then
				local spellStack = select(13,...)
				local spell = procs.combatlog[spellID]
				local expr = GetTime() + spell.dur
				local _,_,icon = GetSpellInfo(tonumber(spellID))
				local sdb = db.spells
				
				if ((sdb[spellID] == nil) or (sdb[spellID].enabled == nil) or (sdb[spellID].enabled == true))
				then
					local spellcolor = spell.color or {1, 0, 0}
					if ((sdb[spellID] ~= nil) and (sdb[spellID].color ~= nil)) then
						spellcolor = sdb[spellID].color
					end
					--self:createCustomTimer(spell.name, spell.dur, expr, tonumber(spellID), spellcolor, false, "", "combatlog")
					self:checkSpellStack(spell.name, spellStack, spell.dur, expr, tonumber(spellID), spellcolor, "", "combatlog")
				end
			end
		elseif (eventType == "SPELL_AURA_REMOVED") then
			if (procs ~= nil) and (procs.combatlog ~= nil) and (procs.combatlog[spellID] ~= nil) then
				for k,v in pairs(ex) do
					if (v.spellname == spellname) and (v.spelltype == "combatlog") then
						ex[k] = del(ex[k])
						Mirror:SendMessage("Quartz3Mirror_UpdateCustom")
					end
				end
			end
		end
	elseif (sourceGUID == UnitGUID("player")) then --check if the player is casting the spell
		local spellID, spellname, spellSchool = select(9, ...)
		if (spellID ~= nil) and (spellID ~= "") then spellID = tostring(spellID) end
			
		if (eventType == "SPELL_CAST_SUCCESS") or (eventType == "SPELL_AURA_APPLIED") then
			if (procs ~= nil) and (procs.combatlog ~= nil) and (procs.combatlog[spellID] ~= nil)
				and (procs.combatlog[spellID].cast ~= nil) and (procs.combatlog[spellID].cast == true)
			then
				local spell = procs.combatlog[spellID]
				local expr = GetTime() + spell.dur
				local _,_,icon = GetSpellInfo(tonumber(spellID))
				local sdb = db.spells

				if ((sdb[spellID] == nil) or (sdb[spellID].enabled == nil) or (sdb[spellID].enabled == true))
				then
					local sound = "None"
					if ((sdb[spellID] ~= nil) and (sdb[spellID].sound ~= nil)) then
						sound = sdb[spellID].sound
					end
					
					local spellcolor = spell.color or {1, 0, 0}
					if ((sdb[spellID] ~= nil) and (sdb[spellID].color ~= nil)) then
						spellcolor = sdb[spellID].color
					end
					
					self:createCustomTimer(spell.name, spell.dur, expr, tonumber(spellID), spellcolor, false, sound, "combatlog")
				end
			end
		
		elseif (eventType == "SPELL_AURA_REMOVED") then
			if (procs ~= nil) and (procs.combatlog ~= nil) and (procs.combatlog[spellID] ~= nil) then
				for k,v in pairs(ex) do
					if (v.spellid == tonumber(spellID)) and (v.spelltype == "combatlog") then
						ex[k] = del(ex[k])
						Mirror:SendMessage("Quartz3Mirror_UpdateCustom")
					end
				end
			end
		end
	end
end

function QuartzProcs:CheckBuffs()
	for i = 1, 40 do
		local name,rank,icon,count,dtype,dur,expr,caster,isStealable,shouldConsolidate,spellId=UnitAura("player",i)
		if not name then break end
		
		if (procs[name] ~= nil) or (procs[tostring(spellId)] ~= nil) then
			local spellname
			local spellkey
			if (procs[tostring(spellId)] ~= nil) then
				spellkey = tostring(spellId)
				spell = procs[spellkey]
				spellname = spell.name
			else
				spellkey = name
				spell = procs[spellkey]
				spellname = name
			end
			
			local sdb = db.spells
			
			if ((sdb[spellkey] == nil) or (sdb[spellkey].enabled == nil) or (sdb[spellkey].enabled == true))
			then
				if (dur ~= nil) and (dur > 0) then
					-- create timer
					local pColor = spell.color
					if (sdb[spellkey] ~= nil) and (sdb[spellkey].color ~= nil) then
						pColor = sdb[spellkey].color
					end
					
					local pSound = "None"
					if (sdb[spellkey] ~= nil) and (sdb[spellkey].sound ~= nil) then
						pSound = sdb[spellkey].sound
					end
						
					if (count) and (count > 0) then
						self:checkSpellStack(spellname, count, dur, expr, tonumber(spellId), pColor, pSound, "aura")
					elseif (ex[spellname].expires == nil) then
						self:createCustomTimer(spellname, dur, expr, tonumber(spellId), pColor, false, pSound, "aura")
						
						-- create cooldown timer
						if (spell.cooldown ~= nil) and (type(spell.cooldown) == "number") and (spell.cooldown > 0) then
							self:createCustomTimer(spellname .. " CD", spell.cooldown, 0, tonumber(spellId), {1,0,0}, true, "", "aura")
						end
					-- update timer
					elseif (ex[spellname].expires ~= expr) then
						self:createCustomTimer(spellname, dur, expr, tonumber(spellId), pColor, false, "", "aura")
					end
				end
			end
		end
	end
	
	for k,v in pairs(ex) do
		local remSpell = true

		if (v.spelltype == "aura") then
			for i = 1, 40 do
				local name,rank,icon,count,dtype,dur,expr,caster,isStealable,shouldConsolidate,spellId=UnitAura("player",i)
				if not name then break end
				
				-- check if cooldown is solar, and check if lunar eclipse exists
				if (v.cooldown == true) and (v.spellid == 48517) and (spellId == 48518) then
					remSpell = true
					break
				-- check if cooldown is lunar, and check if solar eclipse exists
				elseif (v.cooldown == true) and (v.spellid == 48518) and (spellId == 48517) then
					remSpell = true
					break
				elseif (v.cooldown == true) then
					remSpell = false
				elseif (v.cooldown == false) and (v.spellid == spellId) then
					remSpell = false
					break
				end
			end

			if (remSpell == true) then
				if (ex[k] ~= nil) then
					ex[k] = del(ex[k])
					Mirror:SendMessage("Quartz3Mirror_UpdateCustom")
				end
			end
		end
	end
end

function QuartzProcs:refreshSpell(spellname, duration, sExpires, spellid, spellcolor, spellsound, spelltype)
	local foundSpell = false
	for k,v in pairs(ex) do
		if (v.cooldown == false) and (v.spellname == spellname)
		then
			--if (ex[k] ~= nil) then
				--ex[k] = del(ex[k])
				--Mirror:SendMessage("Quartz3Mirror_UpdateCustom")
			--end
			
			self:createCustomTimer(k, duration, sExpires, spellid, spellcolor, false, "", spelltype)
			foundSpell = true
			break
		end
	end
	if (not foundSpell) then
		self:createCustomTimer(spellname, duration, sExpires, spellid, spellcolor, false, spellsound, spelltype)
	end
end

function QuartzProcs:checkSpellStack(spellname, spellcount, duration, sExpires, spellid, spellcolor, spellsound, spelltype)
	local dname = spellname.." ("..spellcount..")"
	
	if (ex[dname].expires == nil) then
		local foundSpell = false
		for k,v in pairs(ex) do
			if (v.cooldown == false) and (v.spellname == spellname) and (k ~= dname)
			then
				if (ex[k] ~= nil) then
					ex[k] = del(ex[k])
					Mirror:SendMessage("Quartz3Mirror_UpdateCustom")
				end
				
				self:createCustomTimer(dname, duration, sExpires, spellid, spellcolor, false, spellsound, spelltype)
				foundSpell = true
				break
			end
		end
		if (not foundSpell) then
			self:createCustomTimer(dname, duration, sExpires, spellid, spellcolor, false, spellsound, spelltype)
		end
	elseif (ex[dname].expires ~= sExpires) then
		self:createCustomTimer(dname, duration, sExpires, spellid, spellcolor, false, spellsound, spelltype)
	end
end

function QuartzProcs:createCustomTimer(spellname, duration, sExpires, spellid, spellcolor, isCoolDown, spellsound, stype)
	local sdb = db.spells[spellname]
	if (sdb == nil) then sdb = db.spells[spellid] end
	
	if (sdb == nil) or ((sdb ~= nil) and ((sdb.enabled == nil) or (sdb.enabled == true))) then
		local sIcon = ''
		local sColor = spellcolor
		if (sdb ~= nil) and (not isCoolDown) then sColor = sdb.color or spellcolor end
		if (type(spellid) == "number") and (spellid > 0) and (isCoolDown == false) then
			_,_,sIcon = GetSpellInfo(spellid)
		end
		
		ex[spellname] = {
			startTime = GetTime(),
			endTime = GetTime() + duration,
			icon = sIcon,
			color = sColor,
			spellid = spellid,
			spellname = GetSpellInfo(spellid),
			expires = sExpires,
			cooldown = isCoolDown,
			spelltype = stype,
		}
		Mirror:SendMessage("Quartz3Mirror_UpdateCustom")
		
		if (string.lower(spellsound) ~= "none") and (spellsound ~= "") then
			PlaySoundFile(spellsound)
		end

		if (not isCoolDown) then
			ex[spellname].startTime = GetTime() - ((GetTime() + duration) - sExpires)
			ex[spellname].endTime = sExpires
		end
	end
end




--[[
*********************************************
*** Configuration ***************************
*********************************************
]]

do
	local cTmpName = ""
	local cTmpColor = {0,0,0}
	local soundslist
	local options

	function cAddBtnFunc()
		if (cTmpName ~= nil) and (cTmpName ~= "") then
			local tmpTbl = {
				name = cTmpName,
				color = cTmpColor,
				ptype = 'custom',
			}
			db.spells[cTmpName] = {
				name = cTmpName,
				color = cTmpColor,
				ptype = 'custom',
			}
			
			procs[cTmpName] = tmpTbl
			
			options.args.customprocs.args[cTmpName] = {
				type = 'group',
				name = cTmpName,
				desc = cTmpName,
				arg = tostring(cTmpName),
				args = spellCustomOptions,
			}
			
			print("Added '"..cTmpName.."'")
			
			cTmpName = ""
		else
			print("Some fields were empty or invalid!")
		end
	end
	
	function getOptions()
		if not soundslist then
			if not media:Fetch("sound", "Meow", true) then
				media:Register("sound", "Meow", [[Interface\AddOns\Quartz_Procs\media\kitten19.ogg]])
			end
			if not media:Fetch("sound", "Warning", true) then
				media:Register("sound", "Warning", [[Interface\AddOns\Quartz_Procs\media\warn1.wav]])
			end
			if not media:Fetch("sound", "System Failure", true) then
				media:Register("sound", "System Failure", [[Interface\AddOns\Quartz_Procs\media\system_failure.mp3]])
			end
			
			soundslist = {
				["None"] = "None", 
				["Sound\\Spells\\Sunwell_Fel_PortalStand.wav"] = "Fel Portal", 
				["Sound\\Spells\\SeepingGaseous_Fel_Nova.wav"] = "Fel Nova", 
				["Sound\\Doodad\\Hellfire_Raid_FX_Explosion05.wav"] = "Explosion",
				["Sound\\Doodad\\KharazahnBellToll.wav"] = "Kara Bell Toll", 
				["Sound\\Interface\\AlarmClockWarning1.wav"] = "Alarm 1", 
				["Sound\\Interface\\AlarmClockWarning2.wav"] = "Alarm 2", 
				["Sound\\Interface\\AlarmClockWarning3.wav"] = "Alarm 3", 
				["Sound\\Spells\\YarrrrImpact.wav"] = "Yarrr",
			}
			for idx, sound in next, media:List("sound") do
				if (string.lower(sound) ~= "none") then
					soundslist[media:Fetch("sound", sound)] = sound
				end
			end
		end
		
		if not spellCustomOptions then
			spellCustomOptions = {
				cToggle = {
					type = 'toggle',
					name = 'Enabled',
					desc = 'Enable/Disable',
					get = function(info)
						local sKey = info[3]
						if (db.spells[sKey] ~= nil) then
							if (db.spells[sKey].enabled ~= nil) then
								return db.spells[sKey].enabled
							else
								return true
							end
						end
						return true
					end,
					set = function(info, v)
						local sKey = info[3]
						if (db.spells[sKey] ~= nil) then
							db.spells[sKey].enabled = v
						end
						--if (v == false) then
							--ProcGroup:RemoveTimer(spellname)
						--end
					end,
					order = 10,
				},
				
				cColor = {
					type = 'color',
					name = 'color',
					get = function(info)
						local sKey = info[3]
						if (db.spells[sKey] ~= nil) then
							return unpack(db.spells[sKey].color)
						end
						return {1,0,0}
					end,
					set = function(info, ...)
						local sKey = info[3]
						if (db.spells[sKey] ~= nil) then
							db.spells[sKey].color = {...}
						end
					end,
					order = 30,
				},

				cSpellSound = {
					type = 'select',
					name = 'Sound',
					desc = 'Sound to make when proc occurs',
					get = function(info)
						local sKey = info[3]
						if (db.spells[sKey] ~= nil) and (db.spells[sKey].sound ~= nil) then
							return db.spells[sKey].sound
						end
						return "None"
					end,
					set = function(info, value)
						local sKey = info[3]
						if (db.spells[sKey] ~= nil) then
							db.spells[sKey].sound = value
							PlaySoundFile(value)
						else
							db.spells[sKey] = { sound = value }
							PlaySoundFile(value)
						end
					end,
					order = 40,
					values = soundslist,
				},
				--[[
				
				cIntCooldown = {
					type = 'toggle',
					name = 'Cooldown',
					desc = 'Track the internal cooldown or the actual cooldown of the proc',
					get = function(info)
						local sKey = info[3]
						if (db.profile.custom[sKey].cd == true) then
							return true
						end
						return false
					end,
					set = function(info, value)
						local sKey = info[3]
						if (db.profile.custom[sKey] ~= nil) then
							db.profile.custom[sKey].cd = value
						end
					end,
					order = 41,
				},
				
				cIntCooldownTime = {
					type = 'input',
					name = 'Cooldown Time',
					desc = 'How long the cooldown lasts',
					validate = function(info, value) return type(tonumber(value)) == "number" end,
					get = function(info)
						local sKey = info[3]
						return tostring(db.profile.custom[sKey].cooldown) or "0"
					end,
					set = function(info, value)
						local sKey = info[3]
						if (db.profile.custom[sKey] ~= nil) then
							db.profile.custom[sKey].cooldown = tonumber(value)
						end
					end,
					hidden = function(info)
						local sKey = info[3]
						if (db.profile.custom[sKey].cd == nil) or (db.profile.custom[sKey].cd == false)
						then return true; end
						return false
					end,
					order = 42,
				},
				]]--

				cSplit = {
					type = 'header',
					name = '',
					order = 99,
				},
				cDelBtn = {
					type = 'execute',
					name = 'Delete',
					func = function(info)
						local sKey = info[3]
						local spellname = sKey
						if (db.spells[sKey] ~= nil) and (db.spells[sKey].name ~= nil) then
							spellname = db.spells[sKey].name
						end
						
						print("Removed custom proc ("..spellname..")")
						db.spells[sKey] = nil
						options.args.customprocs.args[sKey] = nil
						procs[tostring(sKey)] = nil
					end,
					order = 100,
				},
			}
		end
		
		if not options then
			options = {
				type = "group",
				name = "Procs",
				order = 700,
				--set = setOpt,
				--get = getOpt,
				childGroups = "tab",
				args = {
					toggle = {
						type = "toggle",
						name = "Enable",
						desc = "Enable or Disable this module",
						get = function()
							return Quartz3:GetModuleEnabled(MODNAME)
						end,
						set = function(info, v)
							Quartz3:SetModuleEnabled(MODNAME, v)
						end,
						order = 1,
						width = "full",
					},
					mconfig = {
						type = 'group',
						name = 'Settings',
						desc = 'Timer Bar Settings',
						args = {
							mhelp = {
								type = 'description',
								name = 'Quartz: Procs uses the Mirror module to display each proc.' ..
									' In order to change the position/size/texture/font etc., you can '..
									'open the Mirror module settings by clicking the button below, or '..
									'navigate to the Mirror module configuration under the Quartz category.',
								order = 1,
							},
							mBtnOpen = {
								type = 'execute',
								name = 'Mirror Module Options',
								desc = 'Configure what the timer bars will look like',
								func = function()
									if (Quartz3.optFrames.Mirror ~= nil) then
										InterfaceOptionsFrame_OpenToCategory(Quartz3.optFrames.Mirror)
									else
										print("Mirror options are nil!")
									end
								end,
								order = 5,
							},
						},
					},
					classprocs = {
						type = "group",
						name = "Class Procs",
						args = {},
					},
					customprocs = {
						type = "group",
						name = "Custom Procs",
						args = {
							AddNewProc = {
								type = 'group',
								name = "Add Proc",
								desc = "Add a New Custom Proc",
								order = 1,
								args = {
									customName = {
										type = 'input',
										name = 'Name',
										get = function()
											return cTmpName or ''
										end,
										set = function(info, v)
											cTmpName = v
										end,
										order = 1,
									},
									customColor = {
										type = 'color',
										name = 'color',
										get = function()
											return unpack(cTmpColor)
										end,
										set = function(info, ...)
											cTmpColor = {...}
										end,
										order = 5,
									},
									customAddBtn = {
										type = 'execute',
										name = 'Add',
										func = cAddBtnFunc,
										order = 10,
									},
									customResetBtn = {
										type = 'execute',
										name = 'Reset',
										func = function(info)
											cTmpName = ""
										end,
										order = 15,
									},
								},
							},
						},
					},
				}
			}
		end
		
		
		for spellkey,spell in pairs(procs) do
			if (spell.ptype == nil) and (spell.color ~= nil) then --spell.color check prevents combatlog from showing up (or spells we forget to set a color to)
				options.args.classprocs.args[spellkey] = {
					type = 'group',
					name = spell.name or spellkey,
					desc = spell.name or spellkey,
					order = 5,
					arg = tostring(spellkey),
					args = {
						cTitle = {
							type = 'description',
							order = 6,
							name = spell.name or spellkey,
						},
						cSplit = {
							type = "header",
							name = "",
							order = 7,
						},
						cToggle = {
							type = 'toggle',
							name = 'Enabled',
							desc = 'Enable/Disable',
							get = function(info)
								local sKey = info[3]
								if (db.spells[sKey] ~= nil) then
									if (db.spells[sKey].enabled ~= nil) then
										return db.spells[sKey].enabled
									else
										return true
									end
								end
								return true
							end,
							set = function(info, v)
								local sKey = info[3]
								if (db.spells[sKey] ~= nil) then
									db.spells[sKey].enabled = v
								else
									db.spells[sKey] = { enabled = v }
								end
							end,
							order = 10,
						},
						
						cColor = {
							type = 'color',
							name = 'Color',
							desc = "Color of the bar for the timer",
							get = function(info)
								local sKey = info[3]
								if (db.spells[sKey] ~= nil) and (db.spells[sKey].color ~= nil) then
									return unpack(db.spells[sKey].color)
								end
								return unpack(spell.color)
							end,
							set = function(info, ...)
								local sKey = info[3]
								if (db.spells[sKey] ~= nil) then
									db.spells[sKey].color = {...}
								else
									db.spells[sKey] = { color = {...} }
								end
							end,
							order = 30,
						},
						
						cSpellSound = {
							type = 'select',
							name = 'Sound',
							desc = 'Sound to make when proc occurs',
							get = function(info)
								local sKey = info[3]
								if (db.spells[sKey] ~= nil) and (db.spells[sKey].sound ~= nil) then
									return db.spells[sKey].sound
								end
								return "None"
							end,
							set = function(info, value)
								local sKey = info[3]
								if (db.spells[sKey] ~= nil) then
									db.spells[sKey].sound = value
									PlaySoundFile(value)
								else
									db.spells[sKey] = { sound = value }
									PlaySoundFile(value)
								end
							end,
							order = 40,
							values = soundslist,
						},
					},
				}
			end
			
			if (procs.combatlog ~= nil) then
				for k,v in pairs(procs.combatlog) do
					options.args.classprocs.args[k] = {
						type = 'group',
						name = v.name or k,
						desc = v.name or k,
						order = 5,
						arg = tostring(k),
						args = {
							cTitle = {
								type = 'description',
								order = 6,
								name = v.name or k,
							},
							cSplit = {
								type = "header",
								name = "",
								order = 7,
							},
							cToggle = {
								type = 'toggle',
								name = 'Enabled',
								desc = 'Enable/Disable',
								get = function(info)
									local sKey = info[3]
									if (db.spells[sKey] ~= nil) then
										if (db.spells[sKey].enabled ~= nil) then
											return db.spells[sKey].enabled
										else
											return true
										end
									end
									return true
								end,
								set = function(info, v)
									local sKey = info[3]
									if (db.spells[sKey] ~= nil) then
										db.spells[sKey].enabled = v
									else
										db.spells[sKey] = { enabled = v }
									end
								end,
								order = 10,
							},
							
							cColor = {
								type = 'color',
								name = 'Color',
								desc = "Color of the bar for the timer",
								get = function(info)
									local sKey = info[3]
									if (db.spells[sKey] ~= nil) and (db.spells[sKey].color ~= nil) then
										return unpack(db.spells[sKey].color)
									end
									return unpack(v.color)
								end,
								set = function(info, ...)
									local sKey = info[3]
									if (db.spells[sKey] ~= nil) then
										db.spells[sKey].color = {...}
									else
										db.spells[sKey] = { color = {...} }
									end
								end,
								order = 30,
							},
							
							cSpellSound = {
								type = 'select',
								name = 'Sound',
								desc = 'Sound to make when proc occurs',
								get = function(info)
									local sKey = info[3]
									if (db.spells[sKey] ~= nil) and (db.spells[sKey].sound ~= nil) then
										return db.spells[sKey].sound
									end
									return "None"
								end,
								set = function(info, value)
									local sKey = info[3]
									if (db.spells[sKey] ~= nil) then
										db.spells[sKey].sound = value
										PlaySoundFile(value)
									else
										db.spells[sKey] = { sound = value }
										PlaySoundFile(value)
									end
								end,
								order = 40,
								values = soundslist,
							},
						},
					}
				end
			end
		end
		
		for spellkey,spell in pairs(db.spells) do
			if (spell.ptype ~= nil) and (spell.ptype == "custom") then
				options.args.customprocs.args[spellkey] = {
					type = 'group',
					name = spellkey,
					desc = spellkey,
					order = 5,
					arg = tostring(spellkey),
					args = spellCustomOptions,
				}
			end
		end
		
		
		return options
	end
end